/* Nessus Attack Scripting Language
 *
 * Copyright (C) 1999 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <includes.h>
#include <nasl_raw.h>
#include "parser.h"
#include "defines.h"
#include "sanitize.h"
#include "function_call.h"
#include "prompt.h"
#include "pkt_utils.h"
#include "boolean.h"
#include "strutils.h"

extern int execute_instruction(struct arglist *, char *);

u_char * copy_variable(var)
 struct arglist var;
{
 u_char * ret = NULL;
 if(var.type & VAR_INT)ret = var.value;
 else if(var.type & VAR_STR)ret = nstrdup(var.value, var.length, 0);
 else if(var.type & VAR_ARRAY){
  struct arglist * src = (struct arglist*)var.value;
  struct arglist * dst = emalloc(sizeof(struct arglist));
  arg_dup(dst, src);
  ret = (u_char*)dst;
  }
 else if(var.type & VAR_PKT)
 {
  ret = emalloc(var.length);
  memcpy(ret, var.value, var.length);
  /*ret = var.value;*/
 }
return(ret);
}
  
int
affect_array_value(globals, var, output)
 struct arglist * globals;
 struct arglist var;
 char * output;
{
 char* t, *t2;
 char* array;
 char* index;
 struct arglist rt;
 struct arglist * a;
 struct arglist * vars = arg_get_value(globals, "variables");
 struct arglist * types = arg_get_value(globals, "variables_types");
 int vtype;
 u_char * copy;
 int type;
 int len;
 
 t = strdup(output);
 array = t;
 index = strchr(t, ARRAY_OPEN);
 index[0]='\0';
 index++;
 t2 = strchr(index, ARRAY_CLOSE);
 t2[0]='\0';
 rt = sanitize_variable(globals, index);
 if(rt.value<0)
 {
  printf("Error ! Non-existing array !\n");
  return(PKT_ERROR(ERR_VAR|ERR_EXIST));
 }
 vtype = (int)arg_get_value(types, array);
 if((type = arg_get_type(vars, array))>=0)
  {
   if((type != ARG_ARGLIST)&&!(vtype & VAR_STR))
   {
    printf("Error ! %s was first declared as a scalar\n", array);
    return(PKT_ERROR(ERR_VAR|ERR_ISSCALAR));
   }
  a = arg_get_value(vars, array);
  len = arg_get_length(vars, array);
  }
 else
  {
   a = emalloc(sizeof(struct arglist));
   arg_add_value(vars, array, ARG_ARGLIST, -1, a);
   len = sizeof(struct arglist);
  }

 copy = copy_variable(var);
 if(vtype & VAR_STR)
 {
  char * s = (char*)a;
  int len;
  int index;
  if(!(rt.type & STR_ALL_DIGIT))
  {
   printf("Error ! %s is not a good index for a string\n", (char*)rt.value);
   return(PKT_ERROR(ERR_VAR|ERR_VALUE));
  }
  index = atoi(rt.value);
  if(index < len)
  	s[atoi(rt.value)]=copy[0];
  else {
  	printf("Warning ! Trying to put data in a too small string\n");
	}
 }
 else
 {
 if(arg_get_type(a, rt.value)>=0)
  {
  char * old = arg_get_value(a, rt.value);
  if(!(rt.type & VAR_INT))free(old);
  arg_set_value(a, rt.value, rt.length, copy);
  }
 else 
  arg_add_value(a, rt.value, ARG_PTR, rt.length, copy);
 
 if(arg_get_type(types, array)>=0)
  a = arg_get_value(types, array);
 else
  {
   a = emalloc(sizeof(struct arglist));
   arg_add_value(types, array, ARG_ARGLIST, -1, a);
  }
 if(arg_get_type(a, rt.value)>=0)
  arg_set_value(a, rt.value, -1, (void*)rt.type);
 else
  arg_add_value(a, rt.value, ARG_INT,-1, (void*)rt.type);
 }
 free(t);
 if(rt.type & VAR_DELETE)free(rt.value);
 return(PKT_ERROR(NO_ERR));
}
  
 
struct arglist
execute_func(globals, var)
 struct arglist * globals;
 char * var;
{
 struct arglist rt;
 rt = function_call(globals, var);
 return(rt);
}



int
affect_var(globals, var, output)
 struct arglist * globals;
 struct arglist var;
 char * output;
{ 
 struct arglist * vars = arg_get_value(globals, "variables");
 struct arglist * types= arg_get_value(globals, "variables_types");
 u_char * copy;
 char * old;
 var.type = var.type - (var.type & VAR_DELETE);
 copy = copy_variable(var);
 old = arg_get_value(vars, output);
 if(old)
 {
  if(!(var.type & VAR_INT))free(old);
  arg_set_value(vars, output, var.length, copy);
  }
 else
  {
  if(!copy){
   copy = emalloc(2);
   copy[0]='0';
   var.type = VAR_STR;
   var.length = 1;
   }
  arg_add_value(vars, output, ARG_PTR, var.length, copy);
 }
 if(arg_get_type(types, output)>=0)
       arg_set_value(types, output, sizeof(int),
	 	(void*)(var.type));
   else
	arg_add_value(types, output, ARG_INT, sizeof(int),
	 	(void*)(var.type));
 return(PKT_ERROR(NO_ERR));
}
  



int
execute_var_affectation(globals, args)
 struct arglist * globals, *args;
{
 char * output = arg_get_value(args, "output");
 struct arglist * types = arg_get_value(globals, "variables_types");
 char * instruction = arg_get_value(args, "instruction");
 struct arglist * a;
 int type;
 int err = 0;

 if(alldigit(output, strlen(output)))
 {
  printf("%s : Error. %s is not an lvalue\n", instruction, output);
  return(PKT_ERROR(ERR_VAR|ERR_LVALUE));
 }
 
 if(VAR_CONST & (int)arg_get_value(types, output))
 {
  printf("%s : Error. %s is a constant\n", instruction, output);
  return(PKT_ERROR(ERR_VAR|ERR_CONST));
 }
 
 a = parse_instruction(instruction);
 
 type = (int)arg_get_value(a, "type");
 
 if(type == SINGLE_ATOM)
 {
  /*
   * May be a function call or a single value
   */
  char * val;
  struct arglist var;
  int len;
 
  val = strdup(arg_get_value(a, "atom"));
  len = arg_get_length(a, "atom")-1;
  if(val[len]==END_INSTRUCTION)val[len]='\0';
  /*if(quoted_strchr(val, START_FUNCTION))
   var = execute_func(globals, val);
  else*/
   var = sanitize_variable(globals, val);
  if(var.type < 0)return(var.type);
  if(strchr(output, ARRAY_OPEN))err = affect_array_value(globals, var, output);
  else err = affect_var(globals, var, output);
  if(err < 0)return(err);
  if(var.type & VAR_DELETE)free(var.value);
  free(val);
 }
 else
 {
  char * x = quoted_strchr(instruction, AFFECTATION);
  char c;
  struct arglist var;
  
  
  execute_instruction(globals, instruction);
  c = x[0];
  x[0] = '\0';
  var = sanitize_variable(globals, instruction);
  if(var.type < 0)return(var.type);
  /*.... affect here, depending on the type */
  err = affect_var(globals, var, output);
  if(err < 0)return(err);
  x[0]=c;
 }
 arg_free_all(a);
 return(PKT_ERROR(NO_ERR));
}
 
int
execute_instruction_block(globals, args)
 struct arglist * globals, *args;
{
 char * orig = arg_get_value(args, "instruction");
 char * buf = orig;
 char * r,*inst;
 int err = 0;
 while((inst = read_buf_instruction(buf, &r)))
  {
   err = execute_instruction(globals, inst);
   free(inst);
   buf = r;
   if(err < 0)break;
  }
 return(err);
}


int
execute_for_loop(globals, args)
 struct arglist * globals, *args;
{
 char * start = arg_get_value(args, "start");
 char * end = arg_get_value(args, "end");
 char * condition = arg_get_value(args, "condition");
 int err = 0;
 err = execute_instruction(globals, start);
 if(err >= 0)
 {
  err = evaluate_boolean(globals, condition);
  if(err < 0)return(err);
 while(err)
 {
  err = execute_instruction_block(globals, args);
  if(err <0)break;
  err = execute_instruction(globals, end);
  if(err <0)break;
  err = evaluate_boolean(globals, condition);
  if(err < 0)return(err);
 }
 }
 return(err);
}

int
execute_while_loop(globals, args)
 struct arglist *globals, *args;
{
 char * condition = arg_get_value(args, "condition");
 int err = 0;
 err = evaluate_boolean(globals, condition);
 if(err<0)return(err);
 while(err)
  {
   err = execute_instruction_block(globals, args);
   if(err < 0)break;
   err = evaluate_boolean(globals, condition);
   if(err < 0)break;
  }
  return(err);
}

int
execute_if_branch(globals, args)
 struct arglist * globals, *args;
{
 char * condition = arg_get_value(args, "condition");
 char * else_i = arg_get_value(args, "else");
 int err = 0;
 err = evaluate_boolean(globals, condition);
 if(err < 0)return(err);
 if(err)
  err = execute_instruction_block(globals, args);
 else if(else_i){
   char * orig = strdup(else_i);
   char * buf = orig;
   char * r,*inst;

   while((inst = read_buf_instruction(buf, &r)))
  {
   err = execute_instruction(globals, inst);
   free(inst);
   buf = r;
   if(err<0)break;
  }
  free(orig);
  }
 return(err);
}

int
execute_single_atom(globals, args)
 struct arglist * globals, *args;
{
 char * func = arg_get_value(args, "atom");
 struct arglist ret;
 bzero(&ret, sizeof(ret));
 if(quoted_strchr(func, START_FUNCTION))ret = function_call(globals, func);
 return(ret.type<0 ? (int)ret.type:0);
}
 

 

int
execute_instruction(globals, inst)
 struct arglist * globals;
 char * inst;
{ 
  int type = -1;
  struct arglist * args;
  int err = 0;

  inst = strdup(inst);

  if((inst[0]==END_INSTRUCTION)&&(inst[1]=='\0'))return(SINGLE_ATOM);
  args = parse_instruction(inst);
  type = (int)arg_get_value(args, "type");
  switch(type)
  {
   case VAR_AFFECTATION :
   	err = execute_var_affectation(globals, args);break;
   case INSTRUCTION_BLOCK :
   	err = execute_instruction_block(globals, args);break;
   case FOR_LOOP :
   	err = execute_for_loop(globals, args);break;
   case WHILE_LOOP :
   	err = execute_while_loop(globals, args);break;
   case IF_BRANCH :
   	err = execute_if_branch(globals, args);break;
   case SINGLE_ATOM :
   	err = execute_single_atom(globals, args);break;
	
  }
  free(inst);
  arg_free_all(args);
  if(err<0)return(-1);
  else return(type);
}
